% This program executes the same operations that the EXCEL file: Change_savings.xls
% Production Function: Cobb-Douglas:
% Y(t)=B*[K(t)^alpha]*[L(t)*G(t)]^(1-alpha)
% Exogenous growth (labor-augmenting technological progress): G(t)=(1+gamma)G(t-1)

% @ Copyright J. Ruiz 2007

clear

% Structural parameters

alpha=0.36;     % output elasticity of capital
n=0.01;         % population growth
s0=0.20;        % initial savings rate
B=3.0;          % aggregate productivity parameter
delta=0.075;    % depreciation rate
sigma=3.0;      % inverse of elasticity of substitution of consumption
beta=0.95;      % time discount factor
gamma=0.0;     % growth rate of technology

% Steady state (variables per unit of effective labor) under the initial
% savings rate
kss0=(s0*B/((1+n)*(1+gamma)-(1-delta)))^(1/(1-alpha));   % stock of physical capital
yss0=B*(kss0^alpha);   % output
css0=(1-s0)*yss0;      % consumption  
iss0=s0*yss0;          % investment
    % utility
    if sigma==1
        uss0=log(css0);
    else
        uss0=((css0^(1-sigma))-1)/(1-sigma);
    end

% new savings rate
s1=0.30;

% Steady state (variables per unit of effective labor) under the new
% savings rate
kss1=(s1*B/((1+n)*(1+gamma)-(1-delta)))^(1/(1-alpha));   % stock of physical capital
yss1=B*(kss1^alpha);   % output
css1=(1-s1)*yss1;     % cosumption  
iss1=s1*yss1;         % investment
    % utility
    if sigma==1
        uss1=log(css1);
    else
        uss1=((css1^(1-sigma))-1)/(1-sigma);
    end

% transition paths 
T=223;

% exact solution
k=[];k=[k;kss0];
c=[];c=[c;css0];
y=[];y=[y;yss0];
i=[];i=[i;iss0];
u=[];u=[u;uss0];
s=[];s=[s;s0];
w=0;
ka=kss0;

for t=1:T,
    if t<11
        kt=(s0/((1+n)*(1+gamma)))*B*(ka^alpha)+((1-delta)/((1+n)*(1+gamma)))*ka;
        ka=kt;
        k=[k;kt];
        yt=B*kt^alpha;
        y=[y;yt];
        ct=(1-s0)*yt;
        c=[c;ct];
        it=s0*yt;
        i=[i;it];
        st=s0;
        s=[s;st];
        if sigma==1
            ut=log(ct);
        else
            ut=((ct^(1-sigma))-1)/(1-sigma);
        end
        u=[u;ut];
        w=w+(beta^t)*ut;
    else
        kt=(s1/((1+n)*(1+gamma)))*B*(ka^alpha)+((1-delta)/((1+n)*(1+gamma)))*ka;
        ka=kt;
        k=[k;kt];
        yt=B*kt^alpha;
        y=[y;yt];
        ct=(1-s1)*yt;
        c=[c;ct];
        it=s1*yt;
        i=[i;it];
        st=s1;
        s=[s;s1];
        if sigma==1
            ut=log(ct);
        else
            ut=((ct^(1-sigma))-1)/(1-sigma);
        end
        u=[u;ut];
        w=w+(beta^t)*ut;
    end
end

trend=1:T+1; trend=trend';
lnYL=log(y)+log(1+gamma).*trend;   %log output per capita
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% linear approximation
kl=[];kl=[kl;kss0];
cl=[];cl=[cl;css0];
yl=[];yl=[yl;yss0];
il=[];il=[il;iss0];
ul=[];ul=[ul;uss0];
wl=0;
ka=kss0;
D=alpha+(1-alpha)*(1-delta)/((1+n)*(1+gamma));

for t=1:T,
    if t<11
        kt=kss0+D*(ka-kss0);
        ka=kt;
        kl=[kl;kt];
        yt=B*kt^alpha;
        yl=[yl;yt];
        ct=(1-s0)*yt;
        cl=[cl;ct];
        it=s0*yt;
        il=[il;it];
        if sigma==1
            ut=log(ct);
        else
            ut=((ct^(1-sigma))-1)/(1-sigma);
        end
        ul=[ul;ut];
        wl=wl+(beta^t)*ut;
    else
        kt=kss1+D*(ka-kss1);
        ka=kt;
        kl=[kl;kt];
        yt=B*kt^alpha;
        yl=[yl;yt];
        ct=(1-s1)*yt;
        cl=[cl;ct];
        it=s1*yt;
        il=[il;it];
        if sigma==1
            ut=log(ct);
        else
            ut=((ct^(1-sigma))-1)/(1-sigma);
        end
        ul=[ul;ut];
        wl=wl+(beta^t)*ut;
    end
end

lnYLl=log(yl)+log(1+gamma).*trend;   %log output per capita
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% quadratic approximation
kq=[];kq=[kq;kss0];
cq=[];cq=[cq;css0];
yq=[];yq=[yq;yss0];
iq=[];iq=[iq;iss0];
uq=[];uq=[uq;uss0];
wq=0;
ka=kss0;
D=alpha+(1-alpha)*(1-delta)/((1+n)*(1+gamma));
F0=-0.5*alpha*(1-alpha)*((1+n)*(1+gamma)-(1-delta))/((1+n)*(1+gamma)*kss0);
F1=-0.5*alpha*(1-alpha)*((1+n)*(1+gamma)-(1-delta))/((1+n)*(1+gamma)*kss1);
for t=1:T,
    if t<11
        kt=kss0+D*(ka-kss0)+F0*(ka-kss0)^2;
        ka=kt;
        kq=[kq;kt];
        yt=B*kt^alpha;
        yq=[yq;yt];
        ct=(1-s0)*yt;
        cq=[cq;ct];
        it=s0*yt;
        iq=[iq;it];
        if sigma==1
            ut=log(ct);
        else
            ut=((ct^(1-sigma))-1)/(1-sigma);
        end
        uq=[uq;ut];
        wq=wq+(beta^t)*ut;
    else
        kt=kss1+D*(ka-kss1)+F1*(ka-kss1)^2;
        ka=kt;
        kq=[kq;kt];
        yt=B*kt^alpha;
        yq=[yq;yt];
        ct=(1-s1)*yt;
        cq=[cq;ct];
        it=s1*yt;
        iq=[iq;it];
        if sigma==1
            ut=log(ct);
        else
            ut=((ct^(1-sigma))-1)/(1-sigma);
        end
        uq=[uq;ut];
        wq=wq+(beta^t)*ut;
    end
end

lnYLq=log(yq)+log(1+gamma).*trend;   %log output per capita
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Exact continuous
kss0c=(B*s0/(n+delta+gamma))^(1/(1-alpha));
yss0c=B*(kss0c^alpha);
css0c=(1-s0)*yss0c;   
iss0c=s0*yss0c;       
    if sigma==1
        uss0c=log(css0c);
    else
        uss0c=((css0c^(1-sigma))-1)/(1-sigma);
    end


kss1c=(B*s1/(n+delta+gamma))^(1/(1-alpha));
yss1c=B*(kss1c^alpha);
css1c=(1-s1)*yss1c;   
iss1c=s1*yss1c;       
    if sigma==1
        uss1c=log(css1c);
    else
        uss1c=((css1c^(1-sigma))-1)/(1-sigma);
    end


kc=[];kc=[kc;kss0];
cc=[];cc=[cc;css0];
yc=[];yc=[yc;yss0];
ic=[];ic=[ic;iss0];
uc=[];uc=[uc;uss0];
k0a=kss0c;

for t=1:T,
    if t<11
        kt=((k0a^(1-alpha)-(s0*B/(n+delta+gamma)))*exp((alpha-1)*...
            (n+delta+gamma)*t)+s0*B/(n+delta+gamma))^(1/(1-alpha));
        kc=[kc;kt];
        yt=B*kt^alpha;
        yc=[yc;yt];
        ct=(1-s0)*yt;
        cc=[cc;ct];
        it=s0*yt;
        ic=[ic;it];
        k0n=kt;
    else
        kt=((k0n^(1-alpha)-(s1*B/(n+delta+gamma)))*exp((alpha-1)*...
            (n+delta+gamma)*(t-10))+s1*B/(n+delta+gamma))^(1/(1-alpha));
        kc=[kc;kt];
        yt=B*kt^alpha;
        yc=[yc;yt];
        ct=(1-s1)*yt;
        cc=[cc;ct];
        it=s1*yt;
        ic=[ic;it];
    end
end

lnYLc=log(yc)+log(1+gamma).*trend;   %log output per capita
gy=[];gyl=[];gyq=[];gyc=[];
for t=1:T-1
    gyt=((y(t+1)-y(t))/y(t))*100;gy=[gy;gyt];
    gylt=((yl(t+1)-yl(t))/yl(t))*100;gyl=[gyl;gylt];
    gyqt=((yq(t+1)-yq(t))/yq(t))*100;gyq=[gyq;gyqt];
    gyct=((yc(t+1)-yc(t))/yc(t))*100;gyc=[gyc;gyct];
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Graphs
figure;
hold on

subplot(3,2,1)
plot(s(1:100));
title('Savings rate');
xlabel('time');

subplot(3,2,2)
plot([k(1:100) kl(1:100) kq(1:100) kc(1:100)]);
title('Stock of capital after a change in savings rate');
legend('exact solution', 'linear approx.', 'quadratic approx.','exact continuous');
xlabel('time');

subplot(3,2,3)
plot([c(1:100) cl(1:100) cq(1:100) cc(1:100)]);
title('Effects on consumption after a change in savings rate');
legend('exact solution', 'linear approx.', 'quadratic approx.','exact continuous');
xlabel('time');

subplot(3,2,4)
plot([y(1:100) yl(1:100) yq(1:100) yc(1:100)]);
title('Effects on Output of a change in savings rate');
legend('exact solution', 'linear approx.', 'quadratic approx.','exact continuous');
xlabel('time');

subplot(3,2,5)
plot([gy(1:100) gyl(1:100) gyq(1:100) gyc(1:100)]);
title('Growth rate of output after a change in savings rate');
legend('exact solution', 'linear approx.', 'quadratic approx.','exact continuous');
xlabel('time');

subplot(3,2,6)
plot([lnYL(1:100) lnYLl(1:100) lnYLq(1:100) lnYLc(1:100)]);
title('Log of output after a change in savings rate');
legend('exact solution', 'linear approx.', 'quadratic approx.','exact continuous');
xlabel('time');

hold off
