% This program executes the same operations that the EXCEL file: Dynamic_inefficiency.xls
% Production Function: Cobb-Douglas:
% Y(t)=B*[K(t)^alpha]*[L(t)*A(t)]^(1-alpha)
% Exogenous growth (labor-augmenting technological progress): A(t)=[1+gamma(A)]A(t-1)

% @ Copyright J. Ruiz 2007

clear

% Structural parameters

alpha=0.36;     % output elasticity of capital
n=0.01;         % population growth
B=5.0;          % aggregate productivity parameter
delta=0.075;    % depreciation rate
sigma=1.0;      % inverse of elasticity os substitution of consumption
beta=0.90;      % time discount factor
gammaA=0.0;     % growth rate of technology

% Steady states (variables per unit of effective labor) for each s

kss=[]; css=[]; yss=[]; uss=[]; wss=[];
s=[];  % initial savings rate
for i=1:99
    si=i/100; s=[s;si];
    kssi=(si*B/((1+n)*(1+gammaA)-(1-delta)))^(1/(1-alpha)); 
    kss=[kss;kssi];
    yssi=B*kssi^alpha; yss=[yss;yssi];
    cssi=(1-si)*yssi; css=[css;cssi];
    if sigma==1
        ussi=log(cssi);
    else
        ussi=(cssi^(1-sigma)-1)/(1-sigma);
    end
    uss=[uss;ussi];
    wssi=ussi*(1/(1-beta));
    wss=[wss;wssi];
end

% Transition paths for each possible initial savings rate
% The savings rate changes to the level of the Golden Rule [s(RG)=alpha]

T=250;
kt=zeros(T,99); ct=zeros(T,99); yt=zeros(T,99); ut=zeros(T,99); 
wt=zeros(99,1);
for i=1:99
    si=i/100; 
    k0=kss(i); y0=yss(i); c0=css(i); 
    if sigma==1
        u0=log(c0);
    else
        u0=(c0^(1-sigma)-1)/(1-sigma);
    end
    w0=u0;
    for t=1:T
        kt(t,i)=(1/((1+n)*(1+gammaA)))*(alpha*B*(k0^alpha)+(1-delta)*k0);
        k0=kt(t,i);
        yt(t,i)=B*kt(t,i)^alpha;
        ct(t,i)=(1-alpha)*yt(t,i);
        if sigma==1
            ut(t,i)=log(ct(t,i));
        else
            ut(t,i)=(ct(t,i)^(1-sigma)-1)/(1-sigma);
        end
        wtt=w0+(beta^t)*ut(t,i);w0=wtt;
    end
    wt(i)=wtt;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Graphs

RG=alpha.*ones(99,1);
Mcss=max(css);
Mwss=max(wss);
mwss=min(wss);
RG1=zeros(99,1);
RG2=zeros(99,1);
for i=1:99
    RG1(i)=(i-1)*Mcss/98;
    RG2(i)=(i-1)*(Mwss-mwss)/98+mwss;
end

figure;
hold on

[AX,H1,H2]=plotyy(s,css,s,kss,'plot');
set(get(AX(1),'Ylabel'),'String','consumption');
set(get(AX(2),'Ylabel'),'String','capital');
title('Consumption and capital');
xlabel('savings rate');
legend([H1,H2],'consumption','capital');
text(0.36,1,'s(Golden Rule)=alpha=0.36');
text(RG,RG1,'I');
Axis('tight');
hold off

figure;
hold on
plot(s,[wss wt]);
title('Time aggregate utility');
xlabel('savings rate');
legend('Remaining at the initial steady state','Under transition to Golden Rule');
text(0.36,-5,'s(Golden Rule)=alpha=0.36');
text(RG,RG2,'I');
Axis('tight');
hold off
        
    
