% This program executes the same calculations as EXCEL files:
% Solow_Stochastic.xls and Solow_deterministic.xls
%
% Production Function: Cobb-Douglas:
% Y(t)=B*[K(t)^alpha]*[N(t)*A(t)]^(1-alpha)
% which allows for a constant B shifting the level of output.
% Exogenous growth (labor-augmenting technological progress): A(t)=[1+gamma(A)]A(t-1)
%
% This program also computes the deterministic version of Solow's model
% provided the "sigmae" parameter is set at zero
%
% Transitions to steady-state from a lefel of k different from steady-state, the
% 'initial condition' lines in the 'exact solution' and 'linear
% approximation' blocks must be changed to something like: k0=100.0
%
% [Copyright: J.Ruiz]

clear

% Structural parameters

alpha=0.36;     % output elasticity of capital
n=0.0;          % population growth
s=0.36;         % savings rate
B=5.0;          % aggregate productivity parameter
delta=0.075;    % depreciation rate
sigma=3.0;      % inverse of the elasticity of substitution of consumption
beta=0.90;      % time discount factor
gammaA=0.00;     % growth rate of technology
rho=0.90;       % autoregressive parameter in productivity shock
%   Set sigmae = .0 to obtain the solution to the deterministic economy
sigmae=0.01;    % standard deviation of the innovation in the autoregressive process for the technology shock

% Steady state (variables per unit of effective labor)

thetass=1.0;
kss=(s*B*thetass/((1+n)*(1+gammaA)-(1-delta)))^(1/(1-alpha));   % stock of physical capital
yss=B*thetass*(kss^alpha);   % output
css=(1-s)*yss;     % cosumption  
iss=s*yss;         % investment
pmgss=alpha*B*thetass*kss^(alpha-1); % marginal productivity

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% transition paths 
T=260;

% exact solution

k0=kss; theta0=thetass; % initial condition

y0=B*theta0*(k0^alpha);c0=(1-s)*y0; i0=s*y0; pmg0=alpha*B*theta0*k0^(alpha-1);

k=[];k=[k;k0];
c=[];c=[c;c0];
y=[];y=[y;y0];
i=[];i=[i;i0];
pmg=[];pmg=[pmg;pmg0];
theta=[];theta=[theta;theta0];

innov=randn(T+100,1)*sigmae;
innov=innov(101:T+100);

for t=1:T,
    thetat=exp(rho*log(theta0)+innov(t)); theta0=thetat; theta=[theta;thetat];
    kt=(s/((1+n)*(1+gammaA)))*B*thetat*(k0^alpha)+((1-delta)/((1+n)*(1+gammaA)))*k0;
    k0=kt;
    k=[k;kt];
    yt=B*thetat*kt^alpha;
    y=[y;yt];
    ct=(1-s)*yt;
    c=[c;ct];
    it=s*yt;
    i=[i;it];
    pmgt=alpha*B*thetat*kt^(alpha-1);
    pmg=[pmg;pmgt];
end

trend=1:T+1; trend=trend';
lnYL=log(y)+log(1+gammaA).*trend;   %log output per capita

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% linear approximation

k0=kss; theta0=thetass; % initial condition

y0=B*theta0*(k0^alpha);c0=(1-s)*y0; i0=s*y0; pmg0=alpha*B*theta0*k0^(alpha-1);

kl=[];kl=[kl;k0];
cl=[];cl=[cl;c0];
yl=[];yl=[yl;y0];
il=[];il=[il;i0];
pmgl=[];pmgl=[pmgl;pmg0];


D1=alpha+(1-alpha)*(1-delta)/((1+n)*(1+gammaA));
D2=(1-(1-delta)/((1+n)*(1+gammaA)))*(kss/thetass);

for t=1:T,
    kt=kss+D1*(k0-kss)+D2*(theta(t)-thetass);
    k0=kt;
    kl=[kl;kt];
    yt=B*theta(t+1)*kt^alpha;
    yl=[yl;yt];
    ct=(1-s)*yt;
    cl=[cl;ct];
    it=s*yt;
    il=[il;it];
    pmgt=alpha*B*theta(t+1)*kt^(alpha-1);
    pmgl=[pmgl;pmgt];
end


lnYLl=log(yl)+log(1+gammaA).*trend;   %log output per capita


gy=[];gyl=[];
for t=1:T
    gyt=((y(t+1)-y(t))/y(t))*100;gy=[gy;gyt];
    gylt=((yl(t+1)-yl(t))/yl(t))*100;gyl=[gyl;gylt];
end

errork=k-kl;
errorc=c-cl;
errory=y-yl;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Graphs

figure;
hold on

subplot(3,2,1)
plot([k(1:100) kl(1:100)]);
title('Stock of capital');
legend('exact solution', 'linear approx.');
xlabel('time');

subplot(3,2,2)
plot([c(1:100) cl(1:100)]);
title('Consumption');
legend('exact solution', 'linear approx.');
xlabel('time');

subplot(3,2,3)
plot([y(1:100) yl(1:100)]);
title('Output');
legend('exact solution', 'linear approx.');
xlabel('time');

subplot(3,2,4)
plot([gy(1:100) gyl(1:100)]);
title('Growth rate of output');
legend('exact solution', 'linear approx.');
xlabel('time');

subplot(3,2,5)
plot([lnYL(1:100) lnYLl(1:100)]);
title('Logged output');
legend('exact solution', 'linear approx.');
xlabel('time');

subplot(3,2,6)
plot([pmg(1:100) pmgl(1:100)]);
title('Marginal productivity / Real interest rate');
legend('exact solution', 'linear approx.');
xlabel('time');

hold off


% sample moments
display '           S a m p l e   m o m e n t s  '

innov=[0;innov];

T=length(k);

V=[k c y i pmg kl cl yl il pmgl];
V1=[innov log(theta) errork errorc errory];
    
    Mmean=mean(V);Mmean1=mean(V1);
    Mmedian=median(V);Mmedian1=median(V1);
    Mstd=std(V);Mstd1=std(V1);
    Mcv=std(V)./mean(V);Mcv1=std(V1)./mean(V1);
        pc=corrcoef(c,y);
        pk=corrcoef(k,y);
        pi=corrcoef(i,y);
        ppmg=corrcoef(pmg,y);
        pcl=corrcoef(cl,yl);
        pkl=corrcoef(kl,yl);
        pil=corrcoef(il,yl);
        ppmgl=corrcoef(pmgl,yl);
        
    Mcorry=[pc(1,2) pk(1,2) pi(1,2) ppmg(1,2) pcl(1,2) pkl(1,2) pil(1,2) ppmgl(1,2)];
    
        pc=corrcoef(c(1:T-1),y(2:T));
        pk=corrcoef(k(1:T-1),y(2:T));
        pi=corrcoef(i(1:T-1),y(2:T));
        ppmg=corrcoef(pmg(1:T-1),y(2:T));
        pcl=corrcoef(cl(1:T-1),yl(2:T));
        pkl=corrcoef(kl(1:T-1),yl(2:T));
        pil=corrcoef(il(1:T-1),yl(2:T));
        ppmgl=corrcoef(pmgl(1:T-1),yl(2:T));
            
    Mcorry1=[pc(1,2) pk(1,2) pi(1,2) ppmg(1,2) pcl(1,2) pkl(1,2) pil(1,2) ppmgl(1,2)];   
    
        pc=corrcoef(c(2:T),y(1:T-1));
        pk=corrcoef(k(2:T),y(1:T-1));
        pi=corrcoef(i(2:T),y(1:T-1));
        ppmg=corrcoef(pmg(2:T),y(1:T-1));
        pcl=corrcoef(cl(2:T),yl(1:T-1));
        pkl=corrcoef(kl(2:T),yl(1:T-1));
        pil=corrcoef(il(2:T),yl(1:T-1));
        ppmgl=corrcoef(pmgl(2:T),yl(1:T-1));
                
    Mcorrym1=[pc(1,2) pk(1,2) pi(1,2) ppmg(1,2) pcl(1,2) pkl(1,2) pil(1,2) ppmgl(1,2)];   
           
        


display '__________________________________________________________________'
display 'steady states'
[css kss yss iss pmgss]    
display '      c         k         y         i         pmgss'
  
display '_______________________________________________________________________________________________________________________ '
display 'means'
Mmean    
display '      c         k         y         i         pmgg      cl         kl         yl         il         pmgl'

display '__________________________________________________________________ '
display 'means'
Mmean1    
display '    innov    log(theta)  errork    errorc     errory'

display '_______________________________________________________________________________________________________________________ '
display 'medians'
Mmedian    
display '      c         k         y         i         pmgg      cl         kl         yl         il         pmgl'

display '__________________________________________________________________ '
display 'medians'
Mmedian1    
display '    innov    log(theta)  errork    errorc     errory'

display '_______________________________________________________________________________________________________________________ '
display 'standard desviations'
Mstd    
display '      c         k         y         i         pmgg      cl         kl         yl         il         pmgl'

display '__________________________________________________________________ '
display 'standard desviations'
Mstd1    
display '    innov    log(theta)  errork    errorc     errory'

display '__________________________________________________________________________________________________________ '
display 'coefficients of variation'
Mcv    
display '      c         k         y         i         pmgg      cl         kl         yl         il         pmgl'

display '__________________________________________________________________ '
display 'coefficients of variation'
Mcv1    
display '    innov    log(theta)  errork    errorc     errory'

display '__________________________________________________________________________________________________________ '
display 'correlations with output'
Mcorry    
display '      c         k         i        pmg       cl         kl         il        pmgl'

display '__________________________________________________________________________________________________________ '
display 'correlations with output(+1)'
Mcorry1    
display '      c         k         i        pmg       cl         kl         il        pmgl'

display '__________________________________________________________________________________________________________ '
display 'correlations with output(-1)'
Mcorrym1
display '      c         k         i        pmg       cl         kl         il        pmgl'

